#define CORE_NAME "mx25_wdt"

#include <autoconf.h>
#include <hermit.h>
#include <arch/system.h>
#include <arch/clock.h>
#include <irq.h>
#include <init.h>

#define IRQ_EPIT2 27

#define WCR_WOE_BIT             (1 << 6)
#define WCR_WDA_BIT             (1 << 5)
#define WCR_SRS_BIT             (1 << 4)
#define WCR_WRE_BIT             (1 << 3)
#define WCR_WDE_BIT             (1 << 2)
#define WCR_WDBG_BIT            (1 << 1)
#define WCR_WDZST_BIT           (1 << 0)

static int mx25_wdt_ping(int irq, void *data)
{
	u32 epit = EPIT2_BASE_ADDR;
	u32 wdog = WDOG_BASE_ADDR;

	trace();

	write16(wdog + WDOG(WSR), 0x5555);
	write16(wdog + WDOG(WSR), 0xAAAA);

	write32(epit + EPIT(SR), 0x01);
	write32(epit + EPIT(LR), 0xffffffff);

	return 0;
}

void mx25_wdt_init(void)
{
	u32 base = EPIT2_BASE_ADDR;

	trace();

	mx25_per_clock_enable(1);
	mx25_ipg_clock_enable(11);

	write32(base + EPIT(CR), 0x0102041a);
	write32(base + EPIT(SR), 0x01);
	write32(base + EPIT(CMPR), 0xffffffff - 1000000); /* 1s timer */

	write32(base + EPIT(CR), 0x0102041f);

	request_irq(IRQ_EPIT2, mx25_wdt_ping, NULL);
}

void mx25_wdt_reset(void)
{
	u32 wdog = WDOG_BASE_ADDR;
	u32 reg;

	reg = read32(wdog + WDOG(WCR));
	reg &= ~(WCR_SRS_BIT | WCR_WDA_BIT);
	reg |= WCR_WDE_BIT;
	write32(wdog + WDOG(WCR), reg);
}

device_initcall(mx25_wdt_init);
